package com.persagy.dmp.report.controller;

import com.persagy.common.date.DateUtils;
import com.persagy.common.web.MapResponse;
import com.persagy.dmp.report.enumeration.DateType;
import com.persagy.dmp.report.service.DataExecutorService;
import com.persagy.dmp.report.service.ReportUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.Date;
import java.util.Map;

@RestController
@RequestMapping("/report/data/executor")
public class DataExecutorController {

    @Autowired
    DataExecutorService dataExecutorService;

    @Autowired
    ReportUtils reportUtils;

    /**
     * 生成对象信息统计报表
     *
     * @param startTime yyyyMMdd
     * @param endTime   yyyyMMdd
     */
    @GetMapping("/rwdObject")
    public MapResponse rwdObject(@RequestParam DateType dateType,
                                 @RequestParam(required = false) String startTime,
                                 @RequestParam(required = false) String endTime) throws Exception {
        if (null == startTime && null == endTime) {
            Map<String, Date> stringDateMap = reportUtils.getDateTime(dateType);
            dataExecutorService.reportRwdObject(dateType, stringDateMap.get("startTimeDate"), stringDateMap.get("endTimeDate"));
            return new MapResponse();
        }
        Date startTimeDate = DateUtils.parse(startTime);
        Date endTimeDate = DateUtils.parse(endTime);
        dataExecutorService.reportRwdObject(dateType, startTimeDate, endTimeDate);
        return new MapResponse();
    }

    /**
     * 生成报警信息统计报表
     *
     * @param startTime yyyyMMdd
     * @param endTime   yyyyMMdd
     */
    @GetMapping("/alarmRecord")
    public MapResponse alarmRecord(@RequestParam DateType dateType,
                                   @RequestParam(required = false) String startTime,
                                   @RequestParam(required = false) String endTime) throws Exception {
        if (null == startTime && null == endTime) {
            Map<String, Date> stringDateMap = reportUtils.getDateTime(dateType);
            dataExecutorService.reportAlarmRecord(dateType, stringDateMap.get("startTimeDate"), stringDateMap.get("endTimeDate"));
            return new MapResponse();
        }
        Date startTimeDate = DateUtils.parse(startTime);
        Date endTimeDate = DateUtils.parse(endTime);
        dataExecutorService.reportAlarmRecord(dateType, startTimeDate, endTimeDate);
        return new MapResponse();
    }
}
