package com.persagy.dmp.report.jms.executor;

import com.persagy.common.date.DateUtils;
import com.persagy.dmp.report.common.json.JsonMapper;
import com.persagy.dmp.report.entity.QReportAlarmRecord;
import com.persagy.dmp.report.entity.ReportAlarmRecord;
import com.persagy.dmp.report.enumeration.DateType;
import com.persagy.dmp.report.jms.MessageExecutor;
import com.persagy.dmp.report.repository.ReportAlarmRecordRepository;
import com.persagy.dmp.report.service.ReportUtils;
import com.querydsl.core.types.dsl.BooleanExpression;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.MapUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @author: Fengyanjie
 * @date: 2020-10-23
 * @verison: 1.0
 */
@Slf4j
@Component
public class AlarmRecordCreateExecutor implements MessageExecutor {

    @Autowired
    private ReportAlarmRecordRepository reportAlarmRecordRepository;

    @Autowired
    private ReportUtils reportUtils;

    @Override
    public void execute(Map<String, Object> message) {
        Map<String, Object> exts = JsonMapper.json2map(message.get("exts").toString());
        Map<String, Object> obj = JsonMapper.json2map(exts.get("obj").toString());

        String itemCode = MapUtils.getLong(obj, "itemCode") + "";
        String projectId = MapUtils.getString(obj, "projectId");
        String category = MapUtils.getString(obj, "category");
        String classCode = MapUtils.getString(obj, "classCode");
        String level = MapUtils.getString(obj, "level");
        String alarmState = MapUtils.getInteger(obj, "state") + "";
        Long time = MapUtils.getLong(obj, "createTime");
        Date createTime = new Date(time);
        String year = DateUtils.format(createTime, DateUtils.PATTERN_yyyy);
        String month = DateUtils.format(createTime, DateUtils.PATTERN_yyyyMM);
        String day = DateUtils.format(createTime, DateUtils.PATTERN_yyyyMMdd);
        String hour = DateUtils.format(createTime, "yyyyMMddHH");

        QReportAlarmRecord qt = QReportAlarmRecord.reportAlarmRecord;
        BooleanExpression ex = qt.itemCode.eq(itemCode).and(qt.projectId.eq(projectId)).and(qt.category.eq(category))
                .and(qt.classCode.eq(classCode)).and(qt.level.eq(level)).and(qt.alarmState.eq(alarmState)).and(qt.createTime.like(year + "%"));
        Iterable<ReportAlarmRecord> all = reportAlarmRecordRepository.findAll(ex);
        List<ReportAlarmRecord> data = new ArrayList<>();
        all.forEach(data::add);
        if (0 == data.size()) {
            ReportAlarmRecord yearEntity = new ReportAlarmRecord();
            yearEntity.setItemCode(itemCode);
            yearEntity.setProjectId(projectId);
            yearEntity.setCategory(category);
            yearEntity.setClassCode(classCode);
            yearEntity.setLevel(level);
            yearEntity.setAlarmState(alarmState);
            yearEntity.setAlarmRecordNum(1L);
            yearEntity.setYear(year);
            yearEntity.setCreateTime(year);
            yearEntity.setDateType(DateType.YEAR);
            data.add(yearEntity);
            ReportAlarmRecord monthEntity = new ReportAlarmRecord(yearEntity);
            monthEntity.setMonth(month.substring(month.length() - 2));
            monthEntity.setCreateTime(month);
            monthEntity.setDateType(DateType.MONTH);
            data.add(monthEntity);
            ReportAlarmRecord dayEntity = new ReportAlarmRecord(monthEntity);
            dayEntity.setDay(day.substring(day.length() - 2));
            dayEntity.setCreateTime(day);
            dayEntity.setDateType(DateType.DAY);
            data.add(dayEntity);
            ReportAlarmRecord hourEntity = new ReportAlarmRecord(dayEntity);
            hourEntity.setHour(hour.substring(hour.length() - 2));
            hourEntity.setCreateTime(hour);
            hourEntity.setDateType(DateType.HOUR);
            data.add(hourEntity);
            reportAlarmRecordRepository.saveAll(data);
            return;
        }
        for (ReportAlarmRecord datum : data) {
            if (year.equals(datum.getCreateTime())) {
                Long alarmRecordNum = datum.getAlarmRecordNum();
                datum.setAlarmRecordNum((alarmRecordNum += 1));
            }
            if (month.equals(datum.getCreateTime())) {
                Long alarmRecordNum = datum.getAlarmRecordNum();
                datum.setAlarmRecordNum((alarmRecordNum += 1));
            }
            if (day.equals(datum.getCreateTime())) {
                Long alarmRecordNum = datum.getAlarmRecordNum();
                datum.setAlarmRecordNum((alarmRecordNum += 1));
            }
            if (hour.equals(datum.getCreateTime())) {
                Long alarmRecordNum = datum.getAlarmRecordNum();
                datum.setAlarmRecordNum((alarmRecordNum += 1));
            }
        }

        reportAlarmRecordRepository.saveAll(data);
    }
}
