package com.persagy.dmp.report.jms.executor;

import com.persagy.common.criteria.JacksonCriteria;
import com.persagy.common.json.JacksonMapper;
import com.persagy.common.web.MapResponse;
import com.persagy.common.web.PagedResponse;
import com.persagy.dmp.report.common.json.JsonMapper;
import com.persagy.dmp.report.enumeration.DateType;
import com.persagy.dmp.report.feign.TaskClient;
import com.persagy.dmp.report.jms.MessageExecutor;
import com.persagy.dmp.report.model.TaskCfgModel;
import com.persagy.dmp.report.service.DataExecutorService;
import com.persagy.dmp.report.service.ReportUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import java.util.Arrays;
import java.util.Date;
import java.util.Map;

/**
 * @author: fengyanjie
 * @date: 2020-11-25
 * @verison: 1.0
 */
@Slf4j
@Component
public class ReportRwdObjectCreateExecutor implements MessageExecutor {
    @Autowired
    private DataExecutorService dataExecutorService;

    @Autowired
    ReportUtils reportUtils;

    @Override
    public void execute(Map<String, Object> message) {
        Map<String, Object> params = JsonMapper.json2map(message.get("params").toString());
        log.info("reportRwdObjectCreate : {}", JacksonMapper.toSimpleJson(params));
        DateType dateType = DateType.getByCode(params.get("dateType").toString());
        if (StringUtils.isEmpty(dateType)) {
            log.error("reportRwdObjectCreate消息中缺少dateType信息");
            return;
        }

        Map<String, Date> stringDateMap = reportUtils.getDateTime(dateType);
        try {
            dataExecutorService.reportRwdObject(dateType, stringDateMap.get("startTimeDate"), stringDateMap.get("endTimeDate"));
            dataExecutorService.reportAlarmRecord(dateType, stringDateMap.get("startTimeDate"), stringDateMap.get("endTimeDate"));
        } catch (Exception exception) {
            exception.printStackTrace();
        }
    }

    @Autowired
    private TaskClient taskClient;

    //    @Scheduled(cron = "0/10 * * * * ?")
    public void test() {
        JacksonCriteria criteria = JacksonCriteria.newInstance();
        criteria.add("id", "2e25858a-2eea-11eb-a703-0242ac110002");
        PagedResponse<TaskCfgModel> query = taskClient.query(criteria);
        TaskCfgModel taskCfgModel = query.getData().get(0);
        MapResponse response = taskClient.sendTaskToRmq("a", "a", "a", "a", "a", Arrays.asList(taskCfgModel.getId()));
        System.out.println();
    }
}
