package com.persagy.dmp.report.service;

import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.common.web.PagedResponse;
import com.persagy.dmp.report.common.criteria.CriteriaUtils;
import com.persagy.dmp.report.common.criteria.JacksonCriteria;
import com.persagy.dmp.report.entity.BaseEntity;
import com.persagy.dmp.report.entity.QReportRwdObject;
import com.persagy.dmp.report.entity.ReportRwdObject;
import com.persagy.dmp.report.enumeration.DateType;
import com.querydsl.core.Tuple;
import com.querydsl.core.types.dsl.BooleanExpression;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

/**
 * @author fengyanjie
 */
@Service
public class ReportRwdObjectService implements BaseService {

    @Override
    public List<BooleanExpression> parse(ObjectNode criteria) {
        List<BooleanExpression> exps = new LinkedList<>();
        QReportRwdObject qt = QReportRwdObject.reportRwdObject;
        exps.addAll(CriteriaUtils.parse(qt.id, criteria.get("id")));
        exps.addAll(CriteriaUtils.parse(qt.projectId, criteria.get("projectId")));
        exps.addAll(CriteriaUtils.parse(qt.objType, criteria.get("objType")));
        exps.addAll(CriteriaUtils.parse(qt.classCode, criteria.get("classCode")));
        exps.addAll(CriteriaUtils.parse(qt.createTime, criteria.get("createTime")));
        exps.addAll(CriteriaUtils.parse(qt.rwdObjectNum, criteria.get("rwdObjectNum")));
        exps.addAll(CriteriaUtils.parse(qt.dateType, criteria.get("dateType"), DateType.class));
        return exps;
    }

    @Override
    public List<BaseEntity> getData(PagedResponse<Tuple> tuplePagedResponse, List<JacksonCriteria.Calculated> calculatedColumns) {
        List<BaseEntity> list = new ArrayList<>();
        QReportRwdObject qt = QReportRwdObject.reportRwdObject;
        tuplePagedResponse.getData().forEach(item -> {
            ReportRwdObject entity = new ReportRwdObject();
            entity.setId(item.get(qt.id));
            entity.setProjectId(item.get(qt.projectId));
            entity.setObjType(item.get(qt.objType));
            entity.setClassCode(item.get(qt.classCode));
            entity.setYear(item.get(qt.year));
            entity.setMonth(item.get(qt.month));
            entity.setDay(item.get(qt.day));
            entity.setHour(item.get(qt.hour));
            entity.setSeason(item.get(qt.season));
            entity.setWeek(item.get(qt.week));
            entity.setCreateTime(item.get(qt.createTime));
            entity.setRwdObjectNum(item.get(qt.rwdObjectNum));
            entity.setDateType(item.get(qt.dateType));
            List<JacksonCriteria.Calculated> calculatedList = new ArrayList<>();
            for (int i = 0; i < calculatedColumns.size(); i++) {
                JacksonCriteria.Calculated calculated = new JacksonCriteria.Calculated();
                if (calculatedColumns.get(i).getColumn().equals("rwdObjectNum")) {
                    calculated = new JacksonCriteria.Calculated();
                    calculated.setColumn(calculatedColumns.get(i).getColumn());
                    calculated.setFunction(calculatedColumns.get(i).getFunction());
                    calculated.setValue(item.get(i, Long.class));
                }
                calculatedList.add(calculated);
            }
            entity.setCalculatedColumns(calculatedList);
            list.add(entity);
        });
        return list;
    }
}
