package com.persagy.dmp.report.service;

import com.persagy.common.web.PagedResponse;
import com.persagy.dmp.report.common.criteria.CriteriaUtils;
import com.persagy.dmp.report.common.criteria.JacksonCriteria;
import com.persagy.dmp.report.common.json.JsonMapper;
import com.persagy.dmp.report.entity.*;
import com.persagy.dmp.report.repository.ReportConfigRepository;
import com.querydsl.core.Tuple;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * @author fengyanjie
 */
@Service
public class ReportService {

    @Autowired
    protected ReportConfigRepository reportConfigRepository;

    @Autowired
    protected CriteriaUtils criteriaUtils;

    public PagedResponse<BaseEntity> query(JacksonCriteria criteria) {
        PagedResponse<BaseEntity> response = new PagedResponse<>();
        Map<String, List<JacksonCriteria.Calculated>> map = new HashMap<>();
        for (JacksonCriteria.Calculated calculatedColumn : criteria.getCalculatedColumns()) {
            ReportConfig reportConfig = getReportConfig(criteria.getGroupbyColumns(), calculatedColumn.getColumn());
            if (null == reportConfig) {
                response.setMessage(calculatedColumn.getColumn() + " 不支持该计算字段，请联系管理员。");
                return response;
            }
            String mappingName = reportConfig.getMappingName();
            List<JacksonCriteria.Calculated> list = map.get(mappingName);
            if (null == list) {
                list = new LinkedList<>();
            }
            list.add(calculatedColumn);
            map.put(mappingName, list);
        }
        setData(map, criteria, response);
        return response;
    }

    public void setData(Map<String, List<JacksonCriteria.Calculated>> map, JacksonCriteria criteria, PagedResponse<BaseEntity> response) {
        List<BaseEntity> dataList = new LinkedList<>();
        long count = 0;
        for (Map.Entry<String, List<JacksonCriteria.Calculated>> entry : map.entrySet()) {
            String key = entry.getKey();
            List<JacksonCriteria.Calculated> value = entry.getValue();
            PagedResponse<Tuple> tuplePagedResponse = null;
            BaseService service = null;
            switch (key) {
                case "QReportAlarmRecord": {
                    service = new ReportAlarmRecordService();
                    tuplePagedResponse = criteriaUtils.groupbyQuery(QReportAlarmRecord.reportAlarmRecord, service::parse, criteria);
                    break;
                }
                case "QReportRwdObject": {
                    service = new ReportRwdObjectService();
                    tuplePagedResponse = criteriaUtils.groupbyQuery(QReportRwdObject.reportRwdObject, service::parse, criteria);
                    break;
                }
                default: {
                    response.setMessage(key + " 该映射文件不存在，请联系管理员。");
                    return;
                }
            }
            count = tuplePagedResponse.getCount();
            List<BaseEntity> data = service.getData(tuplePagedResponse, value);
            setData(dataList, data, criteria.getGroupbyColumns());
        }
        response.setData(dataList);
        response.setCount(count);
    }

    public void setData(List<BaseEntity> dataList, List<BaseEntity> data, Set<String> groupbyColumns) {
        if (0 == dataList.size()) {
            dataList.addAll(data);
            return;
        }
        for (BaseEntity datum : data) {
            for (BaseEntity entity : dataList) {
                boolean bool = true;
                Map<String, Object> dataMap = JsonMapper.json2map(JsonMapper.toSimpleJson(datum));
                Map<String, Object> dataListMap = JsonMapper.json2map(JsonMapper.toSimpleJson(entity));
                for (String groupbyColumn : groupbyColumns) {
                    if (null == dataMap.get(groupbyColumn) || !dataMap.get(groupbyColumn).equals(dataListMap.get(groupbyColumn))) {
                        bool = false;
                    }
                }
                if (bool) {
                    entity.getCalculatedColumns().addAll(datum.getCalculatedColumns());
                }
            }
        }
    }

    public ReportConfig getReportConfig(Set<String> groupbyColumns, String calculatedColumn) {
        QReportConfig qt = QReportConfig.reportConfig;
        Optional<ReportConfig> one = reportConfigRepository.findOne(qt.calculated.eq(calculatedColumn));
        if (!one.isPresent()) {
            return null;
        }
        List<String> groupList = Arrays.asList(one.get().getGroup().split(","));
        if (groupList.containsAll(groupbyColumns)) {
            return one.get();
        }
        return null;
    }
}
